Chromium Code Reviews
chromiumcodereview-hr@appspot.gserviceaccount.com (chromiumcodereview-hr) | Please choose your nickname with Settings | Help | Chromium Project | Gerrit Changes | Sign out
(330)

Unified Diff: chrome/browser/jumplist_win.h

Issue 7538022: Jumplist Bug (Closed) Base URL: http://src.chromium.org/svn/trunk/src/
Patch Set: '' Created 9 years, 4 months ago
Use n/p to move between diff chunks; N/P to move between comments. Draft comments are only viewable by you.
Jump to:
View side-by-side diff with in-line comments
Download patch
« no previous file with comments | « no previous file | chrome/browser/jumplist_win.cc » ('j') | no next file with comments »
Expand Comments ('e') | Collapse Comments ('c') | Show Comments Hide Comments ('s')
Index: chrome/browser/jumplist_win.h
===================================================================
--- chrome/browser/jumplist_win.h (revision 96768)
+++ chrome/browser/jumplist_win.h (working copy)
@@ -12,7 +12,9 @@
#include <vector>
#include "base/memory/ref_counted.h"
+#include "base/synchronization/lock.h"
#include "chrome/browser/history/history.h"
+#include "chrome/browser/history/history_types.h"
#include "chrome/browser/sessions/tab_restore_service.h"
#include "chrome/browser/sessions/tab_restore_service_observer.h"
#include "content/browser/cancelable_request.h"
@@ -97,14 +99,19 @@
// updates a JumpList when a tab is added or removed.
//
// Updating a JumpList requires some file operations and it is not good to
-// update it in a UI thread. To solve this problem, this class posts a
-// task when it actually updates a JumpList. (This task is implemented in an
-// anomynous namespace in "jumplist_win.cc".)
-class JumpList : public TabRestoreServiceObserver {
+// update it in a UI thread. To solve this problem, this class posts to a
+// runnable method when it actually updates a JumpList.
+class JumpList : public TabRestoreServiceObserver,
+ public NotificationObserver,
+ public base::RefCountedThreadSafe<JumpList> {
public:
JumpList();
- ~JumpList();
+ // NotificationObserver implementation.
+ virtual void Observe(int type,
+ const NotificationSource& source,
+ const NotificationDetails& details);
+
// Registers (or unregisters) this object as an observer.
// When the TabRestoreService object notifies the tab status is changed, this
// class automatically updates an application JumpList.
@@ -122,6 +129,14 @@
// is destroyed.
virtual void TabRestoreServiceDestroyed(TabRestoreService* service);
+ // Cancel a pending jumplist update.
+ void CancelPendingUpdate();
+
+ // Terminate the jumplist: cancel any pending updates and remove observer
+ // from TabRestoreService. This must be called before the profile provided
+ // in the AddObserver method is destroyed.
+ void Terminate();
+
// Returns true if the custom JumpList is enabled.
// We use the custom JumpList when we satisfy the following conditions:
// * Chromium is running on Windows 7 and;
@@ -162,29 +177,61 @@
void OnFaviconDataAvailable(HistoryService::Handle handle,
history::FaviconData favicon);
+ // Callback for TopSites that notifies when the "Most
+ // Visited" list is available. This function updates the ShellLinkItemList
+ // objects and send another query that retrieves a favicon for each URL in
+ // the list.
+ void OnMostVisitedURLsAvailable(
+ const history::MostVisitedURLList& data);
+
+ // Runnable method that updates the jumplist, once all the data
+ // has been fetched.
+ void RunUpdate();
+
+ // Helper method for RunUpdate to decode the data about the asynchrounously
+ // loaded icons.
+ void DecodeIconData(const ShellLinkItemList& item_list);
+
private:
+ friend class base::RefCountedThreadSafe<JumpList>;
+ ~JumpList();
+
// Our consumers for HistoryService.
CancelableRequestConsumer most_visited_consumer_;
CancelableRequestConsumer favicon_consumer_;
+ CancelableRequestConsumer topsites_consumer_;
- // The Profile object used for listening its events.
+ // The Profile object is used to listen for events
Profile* profile_;
+ NotificationRegistrar registrar_;
+
// App id to associate with the jump list.
std::wstring app_id_;
// The directory which contains JumpList icons.
FilePath icon_dir_;
// Items in the "Most Visited" category of the application JumpList.
ShellLinkItemList most_visited_pages_;
// Items in the "Recently Closed" category of the application JumpList.
ShellLinkItemList recently_closed_pages_;
// A list of URLs we need to retrieve their favicons.
typedef std::pair<std::string, scoped_refptr<ShellLinkItem> > URLPair;
std::list<URLPair> icon_urls_;
+
+ // Handle of last favicon request used to cancel if a new request
+ // comes in before the current one returns.
+ FaviconService::Handle handle_;
+
+ // Lock for most_visited_pages_, recently_closed_pages_, icon_urls_
+ // as they may be used by up to 3 threads.
+ base::Lock list_lock_;
};
#endif // CHROME_BROWSER_JUMPLIST_WIN_H_
« no previous file with comments | « no previous file | chrome/browser/jumplist_win.cc » ('j') | no next file with comments »
@@ -162,29 +177,61 @@
// The directory which contains JumpList icons.
FilePath icon_dir_;
- // Items in the "Most Visited" category of the application JumpList.
+ // Items in the "Most Visited" category of the application JumpList,
+ // protected by the list_lock_.
ShellLinkItemList most_visited_pages_;
@@ -162,29 +177,61 @@
ShellLinkItemList most_visited_pages_;
- // Items in the "Recently Closed" category of the application JumpList.
+ // Items in the "Recently Closed" category of the application JumpList,
+ // protected by the list_lock_.
ShellLinkItemList recently_closed_pages_;
@@ -162,29 +177,61 @@
ShellLinkItemList recently_closed_pages_;
- // A list of URLs we need to retrieve their favicons.
+ // A list of URLs we need to retrieve their favicons,
+ // protected by the list_lock_.
typedef std::pair<std::string, scoped_refptr<ShellLinkItem> > URLPair;
std::list<URLPair> icon_urls_;

Powered by Google App Engine
This is Rietveld 408576698